#' ---
#' title: "Generate randomized datasets with shuffled class labels"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

import csv
import time
import pickle
import collections
import numpy as np
import pandas as pd
import sklearn.metrics
import matplotlib as mpl
from sklearn.utils import shuffle
from sklearn.ensemble import RandomForestClassifier

mpl.use('agg')

import matplotlib.pyplot as plt
from pylab import *

# Define the makehash function to create a multi-level dictionary to store all prediction metrics.
def makehash():
    return collections.defaultdict(makehash)

measures = makehash()

# Read the entire data file and define the column names.
# Replace all NA values with 0.
Data_Mat = pd.read_csv("../Data/MMU_Truelabels_CTD_AA_DAC_Moran_PAAC_K4mer_K3mer_K5mer_K6mer_Heart_Liver_Kidney_AdGland_Forebrain_Hindbrain_EmbFacPro_Lintestine_Lung_Limb_Midbrain_Ntube_Sintestine_Ovary_Spleen_Stomach_Thymus_All_pairs.csv", header=None, sep = ",", low_memory=True)
Data_Mat.columns = ["Isoform1", "Isoform2", "Label", "CTD", "AA", "DAC", "Moran", "PAAC", "K4mer", "K3mer", "K5mer", "K6mer", "Heart", "Liver", "Kidney", "AdGland", "Forebrain", "Hindbrain", "EmbFacPro", "Lintestine", "Lung", "Limb", "Midbrain", "Ntube", "Sintestine", "Ovary", "Spleen", "Stomach", "Thymus", "All"]
Data_Mat.fillna(value=0, inplace=True)

# Define the feature columns
cols = np.r_[3:30]

# For every iteration, shuffle the entire data matrix, create a training dataset with 1280000 mRNA Isoform pairs, create a testing dataset with 320000 mRNA isoform pairs not included in the training dataset, randomly shuffle the labels of the training and testing datasets, define the random forest model, train the model and capture the prediction metrics.
from random import sample
for i in range(500):
    start = time.time()
    Data_Mat = sklearn.utils.shuffle(Data_Mat)
    Training = Data_Mat[Data_Mat.Label == 1].head(640000)
    Testing = Data_Mat[Data_Mat.Label == 1].tail(160000)
    Training = Training.append(Data_Mat[Data_Mat.Label == 0].head(640000))
    Testing = Testing.append(Data_Mat[Data_Mat.Label == 0].tail(160000))
    trainArr = Training.iloc[:, cols]
    trainRes = sample(list(Training.iloc[:, 2]), k = 1280000)
    testArr = Testing.iloc[:, cols]
    testRes = sample(list(Testing.iloc[:, 2]), k = 320000)
    rf = RandomForestClassifier(n_estimators=100,n_jobs=-1, oob_score=True, random_state=1)
    rf.fit(trainArr, trainRes)
    results = rf.predict(testArr)
    probs = rf.predict_proba(testArr)
    measures['accuracy'][i] = sklearn.metrics.accuracy_score(testRes,results)
    measures['auroc'][i] = sklearn.metrics.roc_auc_score(testRes, probs[:, 1])
    measures['MCC'][i] = sklearn.metrics.matthews_corrcoef(testRes,results)
    measures['precision'][i] = sklearn.metrics.precision_score(testRes,results)
    measures['recall'][i] = sklearn.metrics.recall_score(testRes,results)
    measures['f1'][i] = sklearn.metrics.f1_score(testRes,results)
    end = time.time()
    print("Iteration %(x)s done in %(y)s" % {"x" : i, "y" : (end - start)}, flush=True)

# Create a list containing all the prediction metrics and save it for future reference.
data = [list(measures['accuracy'].values()), list(measures['auroc'].values()), list(measures['MCC'].values()), list(measures['precision'].values()), list(measures['recall'].values()), list(measures['f1'].values()) ]

with open('../Data/MMU_RF_RandomSets_LabelShuffle_Metrics_13Nov18', 'wb') as fp:
    pickle.dump(data, fp)

# Create a boxplot for all the performance metrics.
fig = plt.figure(1, figsize=(9, 10))
ax = fig.add_subplot(111)
# get dictionary returned from boxplot
bp_dict = boxplot(data, vert=False)
for line in bp_dict['medians']:
    # get position data for median line
    x, y = line.get_xydata()[1] # top of median line
    # overlay median value
    text(x, y, '%.3f' % x,
         horizontalalignment='center', fontsize = 16) # draw above, centered


ax.set_yticklabels(['Accuracy', 'AUROC', 'MCC', 'Precision', 'Recall', 'F1'])
ax.get_xaxis().tick_bottom()
ax.get_yaxis().tick_left()
ax.set_xlim([0,1])
plt.xlabel('Metric Value', fontsize = 20)
plt.ylabel('Performance Metric', fontsize = 20)
plt.title('Performance of randomized label shuffled data', fontsize = 20)
# Save the plot.
fig.savefig('../Data/Fig5.png', bbox_inches='tight')
