#' ---
#' title: "Save final models"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

import csv
import pickle
import collections
import numpy as np
import pandas as pd
import sklearn.metrics
import matplotlib as mpl
from sklearn.externals import joblib
from sklearn.ensemble import RandomForestClassifier

mpl.use('agg')

import matplotlib.pyplot as plt
from pylab import *

# Define the makehash function to create a multi-level dictionary to store all prediction metrics.
def makehash():
    return collections.defaultdict(makehash)

measures = makehash()

# Define a dictionary to store the final models.
model = {}

# Read the training and testing files.
# Define a pandas dataframe to store the predictions.
training = pd.read_csv("../Data/MMU_Train_13Nov18.csv.gz",sep=',', header=None)
testing = pd.read_csv("../Data/MMU_Test_13Nov18.csv.gz",sep=',', header=None)
predictions = pd.DataFrame({'Trans1':testing.iloc[: , 0],'Trans2':testing.iloc[: , 1], 'TrueLabel':testing.iloc[:, 2]})

# For each tissue, extract the training and testing feature array and labels, define the random forest model, predict and store predictions and performance metrics.
for cols, colnames in zip(np.r_[3:12, 13:30], np.r_[3:13,14:30], np.r_[3:14,15:30], np.r_[3:15,16:30], np.r_[3:16,17:30], np.r_[3:17,18:30], np.r_[3:18,19:30], np.r_[3:19,20:30], np.r_[3:20,21:30], np.r_[3:21,22:30], np.r_[3:22,23:30], np.r_[3:23,24:30], np.r_[3:24,25:30], np.r_[3:25,26:30], np.r_[3:26,27:30], np.r_[3:27,28:30], np.r_[3:28,29], np.r_[3:30]], ['withoutHeart', 'withoutLiver', 'withoutKidney', 'withoutAdGland', 'withoutForebrain', 'withoutHindbrain', 'withoutEmbFacPro', 'withoutLintestine', 'withoutLung', 'withoutLimb', 'withoutMidbrain', 'withoutNtube', 'withoutSintestine', 'withoutOvary', 'withoutSpleen', 'withoutStomach', 'withoutThymus', 'All']):
    trainArr = training.iloc[:, cols]
    trainRes = training.iloc[:, 2]
    testArr = testing.iloc[:, cols]
    testRes = testing.iloc[:, 2]
    model[colnames] = RandomForestClassifier(n_estimators=100,n_jobs=-1, oob_score=True, random_state=1)
    model[colnames].fit(trainArr, trainRes)
    results = model[colnames].predict(testArr)
    probs = model[colnames].predict_proba(testArr)
    predictions['pred_%s' % colnames] = results
    predictions['prob_%s' % colnames] = probs[:, 1]
    measures['accuracy'][colnames] = sklearn.metrics.accuracy_score(testRes,results)
    measures['auroc'][colnames] = sklearn.metrics.roc_auc_score(testRes, probs[:, 1])
    measures['MCC'][colnames] = sklearn.metrics.matthews_corrcoef(testRes,results)
    measures['precision'][colnames] = sklearn.metrics.precision_score(testRes,results)
    measures['recall'][colnames] = sklearn.metrics.recall_score(testRes,results)
    measures['f1'][colnames] = sklearn.metrics.f1_score(testRes,results)

# Save the predictions to a file for further analysis.
predictions.to_csv('../Data/Predictions_13Nov18.csv')

# Store the metrics when using all the features.
metrics = [measures['accuracy']['All'], measures['auroc']['All'], measures['MCC']['All'], measures['precision']['All'], measures['recall']['All'], measures['f1']['All']]
print(metrics)

# Feed the dictionary with the training and testing datasets, the final model, predictions and the metrics.
# Save the dictionary.
tuple_objects = (training, testing, model, predictions, metrics)
joblib.dump(tuple_objects,"../Data/modelJoblib_13Nov18.pkl", compress = True)

# Calculate precision and recall.
precision, recall, _ = sklearn.metrics.precision_recall_curve(predictions['TrueLabel'], predictions['prob_All'])
average_precision = sklearn.metrics.average_precision_score(predictions['TrueLabel'], predictions['prob_All'])
print("AUPRC is %s:" % average_precision)

# Calculate true positive and false positive rate.
fpr, tpr, _ = sklearn.metrics.roc_curve(predictions['TrueLabel'], predictions['prob_All'], drop_intermediate=False)
roc_auc = sklearn.metrics.roc_auc_score(predictions['TrueLabel'], predictions['prob_All'])
print("AUROC is %s" % roc_auc)

# Plot the Precision-Recall and Receiver Operating Characteristics curve for the testing dataset.
fig = plt.figure(figsize=(10, 10), dpi = 80)
plt.subplot(2,1,1)
plt.step(recall, precision, color='brown', label='PR curve (area = %0.3f)' % average_precision)
plt.xlim([0.0, 1.0])
plt.ylim([0.0, 1.0])
plt.xlabel('Recall', fontsize = 17)
plt.ylabel('Precision', fontsize = 17)
plt.title('Precision-Recall curve for testing set', fontsize = 17)
plt.legend(loc='lower right', fontsize = 12)

plt.subplot(2,1,2)
plt.plot(fpr, tpr, color='brown', label='ROC curve (area = %0.3f)' % roc_auc)
plt.plot([0, 1], [0, 1], color='black', linestyle='--')
plt.xlim([0.0, 1.0])
plt.ylim([0.0, 1.0])
plt.xlabel('False Positive Rate', fontsize = 17)
plt.ylabel('True Positive Rate', fontsize = 17)
plt.title('Receiver Operating Characteristic curve for testing set', fontsize = 17)
plt.legend(loc='lower right', fontsize = 12)

# Save the plot.
plt.tight_layout()
fig.savefig('../Data/FigS2.png')
