#' ---
#' title: "Generate mRNA isoform level tissue-specific RNA-Seq features"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

# Uncomment the following to install the required package first.
#install.packages("data.table")
#install.packages("protr")
#install.packages("tictoc")
#install.packages("parallel")
library(data.table)
library(protr)
library(tictoc)
library(parallel)

# Save the tissue name supplied as argument in the fpkmTissuePairs.sh script
args = commandArgs(trailingOnly=TRUE)

# Read the fasta files.
# Remove the sequences which have non canonical nucleotide or amino acid residues.
# Remove the sequences which have less than 30 amino acids.
# Remove mRNA sequences for which there is no protein sequence and vice-versa.
# Finally, reorder the sequences, so that these are in the same order.
prot <- readFASTA("../protein_modified_header.fa.gz")
cds <- readFASTA("../rna_modified_header.fa.gz")
prot_cds_clean <- prot[-(which(grepl('^[ACGT]+$', cds) == FALSE))]
cds_clean <- cds[-(which(grepl('^[ACGT]+$', cds) == FALSE))]
cds_subfinal <- cds_clean[-(which(grepl('^[ARNDCEQGHILKMFPSTWYV]+$', prot_cds_clean) == FALSE))]
prot_subfinal <- prot_cds_clean[-(which(grepl('^[ARNDCEQGHILKMFPSTWYV]+$', prot_cds_clean) == FALSE))]
prot_final <- prot_subfinal[-which(nchar(prot_subfinal) <= 30)]
cds_final <- cds_subfinal[-which(nchar(prot_subfinal) <= 30)]
rm(prot_cds_clean);rm(cds_clean);rm(prot);rm(cds);rm(prot_subfinal);rm(cds_subfinal)
cds_final1 <- cds_final[which(names(cds_final) %in% names(prot_final))]
prot_final1 <- prot_final[which(names(prot_final) %in% names(cds_final1))]
cds_final1 <- cds_final1[order(names(cds_final1))]
prot_final1 <- prot_final1[order(names(prot_final1))]
prot <- prot_final1
cds <- cds_final1
stopifnot(names(cds) == names(prot)) # check if there is no extra mRNA or protein sequence.

# Define the z-score function.
zscore <- function(x) {
  (1/2)*(log(((1-x)/(1+x)),2))
}

# Read the expression profile created previously using 'Merge_RNASeq.R' script
FPKM <- read.table("../RNASeq_tissue/MMU_FPKM_Transcript.txt", header=T, sep=",", row.names=1)

# Filter mRNA isoforms for which there are both protein and mRNA sequence as well as expression profile.
Transcripts <- intersect(names(cds), rownames(FPKM))
cds <- cds[which(names(cds) %in% Transcripts)]
prot <- prot[which(names(prot) %in% Transcripts)]
FPKM <- FPKM[which(rownames(FPKM) %in% Transcripts),]
FPKM <- FPKM[order(rownames(FPKM)),]

# Read the file containing the mapping between RNA-Seq samples and tissues.
Exp_Type <- read.table("../RNASeq_tissue/Encode_MMU_Experiments_Types.txt", header=T, sep="\t")

# Extract the expression profile for the selected tissue.
myTissue <- FPKM[,Exp_Type$Sample[which(Exp_Type$TissueType == args[1])]]
dim(myTissue)

# Check if the order of the mRNA isoforms is same in protein sequences, mRNA sequences and expression profile.
stopifnot(rownames(FPKM) == names(cds))
stopifnot(rownames(FPKM) == names(prot))
stopifnot(rownames(FPKM) == Transcripts)
stopifnot(Transcripts == names(cds))
stopifnot(Transcripts == names(prot))
stopifnot(names(cds) == names(cds))

# Since many mRNA isoforms have zero FPKM values, we add 1 to all the FPKM values so that there are no errors while taking their log.
# Calculate the pairwise correlation among all mRNA isoforms for the selected tissue
tic("Correlation")
K3merCor <- cor(t(log2(myTissue + 1)))
rm(K3mer);gc();gc()
toc()

# R cannot allocate a vector of ~76000*76000 elements, so we print the pairwise correlation values in multiple smaller parts.
# Because the correlation matrix is symmteric, we only consider the upper triangle of the matrix. Also, we don't need the self-correlation values, so we ignore the diagonal entries.
# We first store the index number, then create a dataframe with the mRNA isoform ids and their correlation values, then limit the significant digits to three and then finally write the dataframe to file.
# We repeat this for all the parts until we have written the entire upper triangle elements of the correlation matrix.
# Note that the file name will automatically update based on the selected tissue.

tic("Write 1")
sig_values <- which(upper.tri(K3merCor, diag = F)[1:6558,], arr.ind=TRUE)
Cor_Edges <- data.frame(rownames(K3merCor[1:6558,])[sig_values[,1]],colnames(K3merCor[1:6558,])[sig_values[,2]],K3merCor[1:6558,][sig_values])
Cor_Edges[,3] <- round(zscore(Cor_Edges[,3]), digits = 3)
fwrite(Cor_Edges , file = "GRCm38.p4_K7mer_pairs1.csv", sep=",",col.names=F,row.names=F, quote=F)
rm(sig_values);rm(Cor_Edges);gc()
toc()

sig_values <- which(upper.tri(K3merCor, diag = F)[6559:14058,], arr.ind=TRUE)
Cor_Edges <- data.frame(rownames(K3merCor[6559:14058,])[sig_values[,1]],colnames(K3merCor[6559:14058,])[sig_values[,2]],K3merCor[6559:14058,][sig_values])
Cor_Edges[,3] <- round(zscore(Cor_Edges[,3]), digits = 3)
fwrite(Cor_Edges , file = "GRCm38.p4_K7mer_pairs2.csv", sep=",",col.names=F,row.names=F, quote=F)
rm(sig_values);rm(Cor_Edges);gc()

sig_values <- which(upper.tri(K3merCor, diag = F)[14059:22558,], arr.ind=TRUE)
Cor_Edges <- data.frame(rownames(K3merCor[14059:22558,])[sig_values[,1]],colnames(K3merCor[14059:22558,])[sig_values[,2]],K3merCor[14059:22558,][sig_values])
Cor_Edges[,3] <- round(zscore(Cor_Edges[,3]), digits = 3)
fwrite(Cor_Edges , file = "GRCm38.p4_K7mer_pairs3.csv", sep=",",col.names=F,row.names=F, quote=F)
rm(sig_values);rm(Cor_Edges);gc()

sig_values <- which(upper.tri(K3merCor, diag = F)[22559:32558,], arr.ind=TRUE)
Cor_Edges <- data.frame(rownames(K3merCor[22559:32558,])[sig_values[,1]],colnames(K3merCor[22559:32558,])[sig_values[,2]],K3merCor[22559:32558,][sig_values])
Cor_Edges[,3] <- round(zscore(Cor_Edges[,3]), digits = 3)
fwrite(Cor_Edges , file = "GRCm38.p4_K7mer_pairs4.csv", sep=",",col.names=F,row.names=F, quote=F)
rm(sig_values);rm(Cor_Edges);gc()

sig_values <- which(upper.tri(K3merCor, diag = F)[32559:45058,], arr.ind=TRUE)
Cor_Edges <- data.frame(rownames(K3merCor[32559:45058,])[sig_values[,1]],colnames(K3merCor[32559:45058,])[sig_values[,2]],K3merCor[32559:45058,][sig_values])
Cor_Edges[,3] <- round(zscore(Cor_Edges[,3]), digits = 3)
fwrite(Cor_Edges , file = "GRCm38.p4_K7mer_pairs5.csv", sep=",",col.names=F,row.names=F, quote=F)
rm(sig_values);rm(Cor_Edges);gc()

sig_values <- which(upper.tri(K3merCor, diag = F)[45059:60558,], arr.ind=TRUE)
Cor_Edges <- data.frame(rownames(K3merCor[45059:60558,])[sig_values[,1]],colnames(K3merCor[45059:60558,])[sig_values[,2]],K3merCor[45059:60558,][sig_values])
Cor_Edges[,3] <- round(zscore(Cor_Edges[,3]), digits = 3)
fwrite(Cor_Edges , file = "GRCm38.p4_K7mer_pairs6.csv", sep=",",col.names=F,row.names=F, quote=F)
rm(sig_values);rm(Cor_Edges);gc()

tic("Write 7")
sig_values <- which(upper.tri(K3merCor, diag = F)[60559:nrow(K3merCor),], arr.ind=TRUE)
Cor_Edges <- data.frame(rownames(K3merCor[60559:nrow(K3merCor),])[sig_values[,1]],colnames(K3merCor[60559:nrow(K3merCor),])[sig_values[,2]],K3merCor[60559:nrow(K3merCor),][sig_values])
Cor_Edges[,3] <- round(zscore(Cor_Edges[,3]), digits = 3)
fwrite(Cor_Edges , file = "GRCm38.p4_K7mer_pairs7.csv", sep=",",col.names=F,row.names=F, quote=F)
toc()
