#!/bin/bash

#' ---
#' title: "Merge all mRNA Isoform level features in a single data matrix"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

## We have all the sequence and tissue-specific expression features in different files. We will now merge all the properties in a single file.
## Since all the sequence features were calculated in a standardized way, we can horizontally concatenate them.
## Similarly, we can also horizontally concatenate the tissue-specific expression features.
## We can now merge both these concatenated feature files.
## We then extract the features for the labelled mRNA isoform pairs for developing the machine learning models.
## In the end we split the large feature file into smaller file chunks for easier processing.

cd ../Data/

##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##
# This section combines the mRNA Isoform pair features generated from mRNA and protein sequences.
paste -d, GRCm38.p4_CTriad_pairs.csv GRCm38.p4_AAC_pairs.csv GRCm38.p4_DAC_pairs.csv GRCm38.p4_Moran_pairs.csv GRCm38.p4_PAAC_pairs.csv GRCm38.p4_K4mer_pairs.csv GRCm38.p4_K3mer_pairs.csv GRCm38.p4_K5mer_pairs.csv GRCm38.p4_K6mer_pairs.csv | awk -F, '{print $1OFS$2OFS$3OFS$6OFS$9OFS$12OFS$15OFS$18OFS$21OFS$24OFS$27}' OFS=, > GRCm38.p4_CTD_AA_DAC_Moran_PAAC_K4mer_K3mer_K5mer_K6mer_pairs.csv
##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##


##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##
# This section combines the mRNA Isoform pair features generated from RNA-Seq expression profile.
paste -d, GRCm38.p4_Heart_pairs.csv GRCm38.p4_Liver_pairs.csv GRCm38.p4_Kidney_pairs.csv GRCm38.p4_AdGland_pairs.csv GRCm38.p4_Forebrain_pairs.csv GRCm38.p4_Hindbrain_pairs.csv GRCm38.p4_EmbFacPro_pairs.csv GRCm38.p4_Lintestine_pairs.csv GRCm38.p4_Lung_pairs.csv GRCm38.p4_Limb_pairs.csv GRCm38.p4_Midbrain_pairs.csv GRCm38.p4_Ntube_pairs.csv GRCm38.p4_Sintestine_pairs.csv GRCm38.p4_Ovary_pairs.csv GRCm38.p4_Spleen_pairs.csv GRCm38.p4_Stomach_pairs.csv GRCm38.p4_Thymus_pairs.csv GRCm38.p4_FPKM_pairs.csv > GRCm38.p4_All_FPKM_pairs.csv
awk -F, '{print $1OFS$2OFS$3OFS$6OFS$9OFS$12OFS$15OFS$18OFS$21OFS$24OFS$27OFS$30OFS$33OFS$36OFS$39OFS$42OFS$45OFS$48OFS$51OFS$54}' OFS=, GRCm38.p4_All_FPKM_pairs.csv > GRCm38.p4_FPKM_Heart_Liver_Kidney_AdGland_Forebrain_Hindbrain_EmbFacPro_Lintestine_Lung_Limb_Midbrain_Ntube_Sintestine_Ovary_Spleen_Stomach_Thymus_All_pairs.csv
rm GRCm38.p4_All_FPKM_pairs.csv
##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##


##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##
# This section combines all mRNA Isoform pair features.
paste -d, GRCm38.p4_CTD_AA_DAC_Moran_PAAC_K4mer_K3mer_K5mer_K6mer_pairs.csv GRCm38.p4_FPKM_Heart_Liver_Kidney_AdGland_Forebrain_Hindbrain_EmbFacPro_Lintestine_Lung_Limb_Midbrain_Ntube_Sintestine_Ovary_Spleen_Stomach_Thymus_All_pairs.csv | awk -F, '{print $1OFS$2OFS$3OFS$4OFS$5OFS$6OFS$7OFS$8OFS$9OFS$10OFS$11OFS$14OFS$15OFS$16OFS$17OFS$18OFS$19OFS$20OFS$21OFS$22OFS$23OFS$24OFS$25OFS$26OFS$27OFS$28OFS$29OFS$30OFS$31}' OFS=, > GRCm38.p4_CTD_AA_DAC_Moran_PAAC_K4mer_K3mer_K5mer_K6mer_Heart_Liver_Kidney_AdGland_Forebrain_Hindbrain_EmbFacPro_Lintestine_Lung_Limb_Midbrain_Ntube_Sintestine_Ovary_Spleen_Stomach_Thymus_All_pairs.csv
##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##


##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##
# When correlation is perfect (1 or -1), this leads to a z-score of Inf. We replace such infinite values with an empirically high value of 100.
# Some transcripts are not expressed in certain tissues, and this leads to a correlation value of 'NA'. We replace these with '0' when making predictions in python.
sed -i 's/Inf/100/g' GRCm38.p4_CTD_AA_DAC_Moran_PAAC_K4mer_K3mer_K5mer_K6mer_Heart_Liver_Kidney_AdGland_Forebrain_Hindbrain_EmbFacPro_Lintestine_Lung_Limb_Midbrain_Ntube_Sintestine_Ovary_Spleen_Stomach_Thymus_All_pairs.csv
##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##


##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##
# This section extracts the labelled mRNA isoform pairs for developing the machine learning models.
awk -F, 'NR==FNR{a[$1,$2]=$3OFS$4OFS$5OFS$6OFS$7OFS$8OFS$9OFS$10OFS$11OFS$12OFS$13OFS$14OFS$15OFS$16OFS$17OFS$18OFS$19OFS$20OFS$21OFS$22OFS$23OFS$24OFS$25OFS$26OFS$27OFS$28OFS$29;next}{print $0, a[$1,$2]}' OFS=, GRCm38.p4_CTD_AA_DAC_Moran_PAAC_K4mer_K3mer_K5mer_K6mer_Heart_Liver_Kidney_AdGland_Forebrain_Hindbrain_EmbFacPro_Lintestine_Lung_Limb_Midbrain_Ntube_Sintestine_Ovary_Spleen_Stomach_Thymus_All_pairs.csv MMU_TrueLabels_12Nov18.txt > GRCm38.p4_Truelabels_CTD_AA_DAC_Moran_PAAC_K4mer_K3mer_K5mer_K6mer_Heart_Liver_Kidney_AdGland_Forebrain_Hindbrain_EmbFacPro_Lintestine_Lung_Limb_Midbrain_Ntube_Sintestine_Ovary_Spleen_Stomach_Thymus_All_pairs.csv
##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##


##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##
# Because the entire data file is very large, we split it into 25 smaller file chunks.
split -l 114990129 -d GRCm38.p4_CTD_AA_DAC_Moran_PAAC_K4mer_K3mer_K5mer_K6mer_Heart_Liver_Kidney_AdGland_Forebrain_Hindbrain_EmbFacPro_Lintestine_Lung_Limb_Midbrain_Ntube_Sintestine_Ovary_Spleen_Stomach_Thymus_All_pairs.csv MMU_Predict_
##xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx##
