#' ---
#' title: "Generate mRNA Isoform pair features using protein sequences"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

# Uncomment the following to install the required package first.
#install.packages("data.table")
#install.packages("protr")
#install.packages("tictoc")
#install.packages("parallel")
#install.packages("rDNAse")
library(data.table)
library(protr)
library(tictoc)
library(parallel)

# Save the function name and number of cores supplied as argument in the protSeqPairs.sh script
args = commandArgs(trailingOnly = TRUE)

# Read the fasta files.
# Remove the sequences which have non canonical nucleotide or amino acid residues.
# Remove the sequences which have less than 30 amino acids.
# Remove mRNA sequences for which there is no protein sequence and vice-versa.
# Finally, reorder the sequences, so that these are in the same order.
prot <- readFASTA("../Data/protein_modified_header.fa.gz")
cds <- readFASTA("../Data/rna_modified_header.fa.gz")
prot_cds_clean <- prot[-(which(grepl('^[ACGT]+$', cds) == FALSE))]
cds_clean <- cds[-(which(grepl('^[ACGT]+$', cds) == FALSE))]
cds_subfinal <-
  cds_clean[-(which(grepl(
    '^[ARNDCEQGHILKMFPSTWYV]+$', prot_cds_clean
  ) == FALSE))]
prot_subfinal <-
  prot_cds_clean[-(which(grepl(
    '^[ARNDCEQGHILKMFPSTWYV]+$', prot_cds_clean
  ) == FALSE))]
prot_final <- prot_subfinal[-which(nchar(prot_subfinal) <= 30)]
cds_final <- cds_subfinal[-which(nchar(prot_subfinal) <= 30)]
rm(prot_cds_clean)
rm(cds_clean)
rm(prot)
rm(cds)
rm(prot_subfinal)
rm(cds_subfinal)
cds_final1 <-
  cds_final[which(names(cds_final) %in% names(prot_final))]
prot_final1 <-
  prot_final[which(names(prot_final) %in% names(cds_final1))]
cds_final1 <- cds_final1[order(names(cds_final1))]
prot_final1 <- prot_final1[order(names(prot_final1))]
prot <- prot_final1
cds <- cds_final1
stopifnot(names(cds) == names(prot)) # check if there is no extra mRNA or protein sequence.

# Define the z-score function.
zscore <- function(x) {
  (1 / 2) * (log(((1 - x) / (1 + x)), 2))
}

# Read the expression profile created previously using 'Merge_RNASeq.R' script.
FPKM <-
  read.table(
    "../Data/MMU_FPKM_Transcript.txt",
    header = T,
    sep = ",",
    row.names = 1
  )

# Filter mRNA isoforms for which there are both protein and mRNA sequence as well as expression profile.
Transcripts <- intersect(names(cds), rownames(FPKM))
cds <- cds[which(names(cds) %in% Transcripts)]
prot <- prot[which(names(prot) %in% Transcripts)]
FPKM <- FPKM[which(rownames(FPKM) %in% Transcripts),]
FPKM <- FPKM[order(rownames(FPKM)),]

# Check if the order of mRNA isoforms is same across the protein sequence, mRNA sequence and expression profile.
stopifnot(rownames(FPKM) == names(cds))
stopifnot(rownames(FPKM) == names(prot))
stopifnot(rownames(FPKM) == Transcripts)
stopifnot(Transcripts == names(cds))
stopifnot(Transcripts == names(prot))
stopifnot(names(cds) == names(cds))

# Define a cluster of user supplied number of cores.
# Export the supplied function to all the cores.
# Calculate the protein sequence property.
# Close the cluster
cl <- makeCluster(args[2])
clusterExport(cl, args[1])
tic('Parallel')
K3mer <- t(parSapply(cl, prot, FUN = args[1]))
dim(K3mer)
toc()
stopCluster(cl)

# Calculate the pairwise correlation among all mRNA isoforms.
K3merCor <- cor(t(K3mer))
rm(K3mer)
gc()
gc()

# R cannot allocate a vector of ~76000*76000 elements, so we print the pairwise correlation values in multiple smaller parts.
# Because the correlation matrix is symmteric, we only consider the upper triangle of the matrix. Also, we don't need the self-correlation values, so we ignore the diagonal entries.
# We first store the index number, then create a dataframe with the mRNA isoform ids and their correlation values, then limit the significant digits to three and then finally write the dataframe to file.
# We repeat this for all the parts until we have written the entire upper triangle elements of the correlation matrix.
tic("Write 1")
sig_values <-
  which(upper.tri(K3merCor, diag = F)[1:6558,], arr.ind = TRUE)
Cor_Edges <-
  data.frame(rownames(K3merCor[1:6558,])[sig_values[, 1]], colnames(K3merCor[1:6558,])[sig_values[, 2]], K3merCor[1:6558,][sig_values])
Cor_Edges[, 3] <- round(zscore(Cor_Edges[, 3]), digits = 3)
fwrite(
  Cor_Edges ,
  file = "GRCm38.p4_K7mer_pairs1.csv",
  sep = ",",
  col.names = F,
  row.names = F,
  quote = F
)
rm(sig_values)
rm(Cor_Edges)
gc()
toc()

sig_values <-
  which(upper.tri(K3merCor, diag = F)[6559:14058,], arr.ind = TRUE)
Cor_Edges <-
  data.frame(rownames(K3merCor[6559:14058,])[sig_values[, 1]], colnames(K3merCor[6559:14058,])[sig_values[, 2]], K3merCor[6559:14058,][sig_values])
Cor_Edges[, 3] <- round(zscore(Cor_Edges[, 3]), digits = 3)
fwrite(
  Cor_Edges ,
  file = "GRCm38.p4_K7mer_pairs2.csv",
  sep = ",",
  col.names = F,
  row.names = F,
  quote = F
)
rm(sig_values)
rm(Cor_Edges)
gc()

sig_values <-
  which(upper.tri(K3merCor, diag = F)[14059:22558,], arr.ind = TRUE)
Cor_Edges <-
  data.frame(rownames(K3merCor[14059:22558,])[sig_values[, 1]], colnames(K3merCor[14059:22558,])[sig_values[, 2]], K3merCor[14059:22558,][sig_values])
Cor_Edges[, 3] <- round(zscore(Cor_Edges[, 3]), digits = 3)
fwrite(
  Cor_Edges ,
  file = "GRCm38.p4_K7mer_pairs3.csv",
  sep = ",",
  col.names = F,
  row.names = F,
  quote = F
)
rm(sig_values)
rm(Cor_Edges)
gc()

sig_values <-
  which(upper.tri(K3merCor, diag = F)[22559:32558,], arr.ind = TRUE)
Cor_Edges <-
  data.frame(rownames(K3merCor[22559:32558,])[sig_values[, 1]], colnames(K3merCor[22559:32558,])[sig_values[, 2]], K3merCor[22559:32558,][sig_values])
Cor_Edges[, 3] <- round(zscore(Cor_Edges[, 3]), digits = 3)
fwrite(
  Cor_Edges ,
  file = "GRCm38.p4_K7mer_pairs4.csv",
  sep = ",",
  col.names = F,
  row.names = F,
  quote = F
)
rm(sig_values)
rm(Cor_Edges)
gc()

sig_values <-
  which(upper.tri(K3merCor, diag = F)[32559:45058,], arr.ind = TRUE)
Cor_Edges <-
  data.frame(rownames(K3merCor[32559:45058,])[sig_values[, 1]], colnames(K3merCor[32559:45058,])[sig_values[, 2]], K3merCor[32559:45058,][sig_values])
Cor_Edges[, 3] <- round(zscore(Cor_Edges[, 3]), digits = 3)
fwrite(
  Cor_Edges ,
  file = "GRCm38.p4_K7mer_pairs5.csv",
  sep = ",",
  col.names = F,
  row.names = F,
  quote = F
)
rm(sig_values)
rm(Cor_Edges)
gc()

sig_values <-
  which(upper.tri(K3merCor, diag = F)[45059:60558,], arr.ind = TRUE)
Cor_Edges <-
  data.frame(rownames(K3merCor[45059:60558,])[sig_values[, 1]], colnames(K3merCor[45059:60558,])[sig_values[, 2]], K3merCor[45059:60558,][sig_values])
Cor_Edges[, 3] <- round(zscore(Cor_Edges[, 3]), digits = 3)
fwrite(
  Cor_Edges ,
  file = "GRCm38.p4_K7mer_pairs6.csv",
  sep = ",",
  col.names = F,
  row.names = F,
  quote = F
)
rm(sig_values)
rm(Cor_Edges)
gc()

tic("Write 7")
sig_values <-
  which(upper.tri(K3merCor, diag = F)[60559:nrow(K3merCor),], arr.ind =
          TRUE)
Cor_Edges <-
  data.frame(rownames(K3merCor[60559:nrow(K3merCor),])[sig_values[, 1]], colnames(K3merCor[60559:nrow(K3merCor),])[sig_values[, 2]], K3merCor[60559:nrow(K3merCor),][sig_values])
Cor_Edges[, 3] <- round(zscore(Cor_Edges[, 3]), digits = 3)
fwrite(
  Cor_Edges ,
  file = "GRCm38.p4_K7mer_pairs7.csv",
  sep = ",",
  col.names = F,
  row.names = F,
  quote = F
)
toc()
