#' ---
#' title: "Perform stratified K-fold validation"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

import numpy as np
import pandas as pd
import sklearn.metrics
from scipy import interp
import matplotlib as mpl
from sklearn.metrics import roc_curve, auc
from sklearn.model_selection import StratifiedKFold
from sklearn.ensemble import RandomForestClassifier

mpl.use('agg')

import matplotlib.pyplot as plt
from pylab import *

# Define the feature columns
cols = np.r_[3:30]

# Read the training file and define the training feature array and training labels.
training = pd.read_csv("../Data/MMU_Train_13Nov18.csv", header=0, sep = ",")
trainArr = training.iloc[:, cols] # The first two columns contain the mRNA isoform IDs.
trainRes = training.iloc[:, 2]

# Define the parameters of stratified K-fold and random forest classifier.
cv = StratifiedKFold(n_splits=10, random_state = 1)
classifier = RandomForestClassifier(n_estimators=100,n_jobs=-1, oob_score=True, random_state=1)

# We will now split the dataset into 10 folds and calculate the precision, recall and the predicted probability scores for each fold and plot the Precision-Recall curve.
fig = plt.figure(figsize=(10, 10), dpi = 80)
plt.subplot(2,1,1)
y_real = []
y_proba = []
for i, (train_index, test_index) in enumerate(cv.split(trainArr, trainRes)):
    Xtrain, Xtest = trainArr.iloc[train_index,:], trainArr.iloc[test_index, :]
    ytrain, ytest = trainRes[train_index], trainRes[test_index]
    classifier.fit(Xtrain, ytrain)
    pred_proba = classifier.predict_proba(Xtest)
    precision, recall, _ = sklearn.metrics.precision_recall_curve(ytest, pred_proba[:,1])
    plt.step(recall, precision, label='Fold %d AUC=%.4f' % (i+1, auc(recall, precision)))
    y_real.append(ytest)
    y_proba.append(pred_proba[:,1])

y_real = np.concatenate(y_real)
y_proba = np.concatenate(y_proba)
precision, recall, _ = sklearn.metrics.precision_recall_curve(y_real, y_proba)
plt.step(recall, precision, label='Overall AUC=%.4f' % (auc(recall, precision)), color='brown')
plt.xlim([0.0, 1.0])
plt.ylim([0.0, 1.0])
plt.xlabel('Recall', fontsize = 17)
plt.ylabel('Precision', fontsize = 17)
plt.title('Precision-Recall curve for 10-fold cross-validation', fontsize = 17)
plt.legend(loc="lower right", fontsize = 12)

# We will now split the dataset into 10 folds and calculate the true positive rate, false positive rate and the predicted probability scores for each fold and plot the Receiver Operating Characteristics curve.
# Note the re-use of the same variable names!
plt.subplot(2,1,2)
y_real = []
y_proba = []
for i, (train_index, test_index) in enumerate(cv.split(trainArr, trainRes)):
    Xtrain, Xtest = trainArr.iloc[train_index,:], trainArr.iloc[test_index, :]
    ytrain, ytest = trainRes[train_index], trainRes[test_index]
    classifier.fit(Xtrain, ytrain)
    pred_proba = classifier.predict_proba(Xtest)
    precision, recall, _ = roc_curve(ytest, pred_proba[:,1])
    plt.plot(precision, recall, label='Fold %d AUC=%.4f' % (i+1, auc(precision, recall)))
    y_real.append(ytest)
    y_proba.append(pred_proba[:,1])

y_real = np.concatenate(y_real)
y_proba = np.concatenate(y_proba)
precision, recall, _ = roc_curve(y_real, y_proba)
plt.plot(precision, recall, label='Overall AUC=%.4f' % (auc(precision, recall)), color='brown')
plt.plot([0, 1], [0, 1], linestyle='--', color='black', label='Random')
plt.xlim([0.0, 1.0])
plt.ylim([0.0, 1.0])
plt.xlabel('False Positive Rate', fontsize = 17)
plt.ylabel('True Positive Rate', fontsize = 17)
plt.title('Receiver operating characteristic curve for 10-fold cross-validation', fontsize = 17)
plt.legend(loc="lower right", fontsize = 12)
# Save the plot to a file.
fig.tight_layout()
fig.savefig('../Data/Fig6.png')
